from typing import Any

class WSGIWarning(Warning): ...
class HTTPWarning(Warning): ...

def check_string(context, obj, stacklevel: int = ...): ...

class InputStream:
    def __init__(self, stream): ...
    def read(self, *args): ...
    def readline(self, *args): ...
    def __iter__(self): ...
    def close(self): ...

class ErrorStream:
    def __init__(self, stream): ...
    def write(self, s): ...
    def flush(self): ...
    def writelines(self, seq): ...
    def close(self): ...

class GuardedWrite:
    def __init__(self, write, chunks): ...
    def __call__(self, s): ...

class GuardedIterator:
    closed: Any
    headers_set: Any
    chunks: Any
    def __init__(self, iterator, headers_set, chunks): ...
    def __iter__(self): ...
    def next(self): ...
    def close(self): ...
    def __del__(self): ...

class LintMiddleware:
    app: Any
    def __init__(self, app): ...
    def check_environ(self, environ): ...
    def check_start_response(self, status, headers, exc_info): ...
    def check_headers(self, headers): ...
    def check_iterator(self, app_iter): ...
    def __call__(self, *args, **kwargs): ...
